# Generic Functions for Node.js #

## 1. Overview ##
Generic Functions for Node.js is an implementation of <a href="http://en.wikipedia.org/wiki/Generic_function">generic functions</a> for Node.js, based on the Common Lisp Object System.

Here's the run-down. A generic function is a callable function that dispatches to different specializations ("methods") for different parameters. Primitive object-oriented systems use single dispatch dispatch only the first argument (usually the first one left to the dot) whereas advanced ones dispatch on all their arguments (which makes dot notation unnecessary).

## 2. Example ##
	var util = require("util");
	var generic = require("generic-function");
	var collide = generic(2);
	collide.addMethod("string", "number", function (x, y) {
		util.log("A "+x+" collided with "+y+" pieces of asteriods.");
	});
	collide.addMethod("number", "string", function (x, y) {
		util.log(""+x+" pieces of asteriods collided with a "+y+".");
	});
	collide("spaceship", 42);
	-> "A spaceship collided with 42 pieces of asteriods."
	collide(23, "space station");
	-> "23 pieces of asteriods collided with a space station."

## 3. Supported Functionality ##
- Generic function dispatch (without walking the prototype hierarch yet)
- Adding methods at run-time
- Removing methods at run-time
- Finding a method with a particular type signature

## 4. Open Questions ##
- Should intrinsic types be dispatched like their object-based counterparts? I.e., should 'string' be dispatched as a 'String' and vice-versa?
- Method dispatch examines leaf object types, but not the prototype hierarchy. Walk the prototype hierarchy to find the matching method.
- Support 0-arg generic functions.
